from typing import ClassVar, cast

from pydantic.dataclasses import dataclass

from cuenca.resources.base import Creatable

from ..exc import CuencaException
from ..http import Session, session as global_session


@dataclass
class LoginToken(Creatable):
    _resource: ClassVar = 'login_tokens'

    login_token: str

    @classmethod
    def create(cls, session: Session = global_session) -> 'LoginToken':
        """
        Use this method to create a token that will last for 7 days
        Make sure to store this token in a safe place

        :return: Token that you can use in cuenca.configure
        """
        if not session.login:
            raise CuencaException('You have to login first')
        return cast('LoginToken', cls._create(session=session))



from typing import TYPE_CHECKING, ClassVar, cast

from cuenca_validations.types import CardFundingType, CardIssuer
from cuenca_validations.types.requests import CardActivationRequest

from cuenca.resources.base import Creatable

from ..http import Session, session as global_session

if TYPE_CHECKING:
    from .cards import Card


class CardActivation(Creatable):
    _resource: ClassVar = 'card_activations'

    @classmethod
    def create(
        cls,
        number: str,
        exp_month: int,
        exp_year: int,
        cvv2: str,
        issuer: CardIssuer,
        funding_type: CardFundingType,
        *,
        session: Session = global_session,
    ) -> 'Card':
        """
        Associates a physical card with the current user

        :param number: Card number
        :param exp_month:
        :param exp_year:
        :param cvv2:
        :param issuer:
        :param funding_type: debit or credit
        """
        req = CardActivationRequest(
            number=number,
            exp_month=exp_month,
            exp_year=exp_year,
            cvv2=cvv2,
            issuer=issuer,
            funding_type=funding_type,
        )
        return cast('Card', cls._create(session=session, **req.dict()))



import datetime as dt
from dataclasses import dataclass
from typing import TYPE_CHECKING

from .exc import InvalidPassword

if TYPE_CHECKING:
    from .http import Session


@dataclass
class Login:
    last_login_at: dt.datetime
    login_id: str

    @classmethod
    def log_in(cls, password: str, session: 'Session') -> 'Login':
        resp = session.post('/log_in', dict(password=password))
        if not resp['success']:
            raise InvalidPassword
        return cls(**resp)

    def log_out(self, session: 'Session') -> None:
        session.post('/log_out', dict())



from typing import ClassVar

from cuenca_validations.types.requests import PasswordRequest
from pydantic.dataclasses import dataclass

from cuenca.resources.base import Creatable

from ..http import Session, session as global_session


@dataclass
class Password(Creatable):
    _resource: ClassVar = 'passwords'

    @classmethod
    def create(
        cls, password: str, *, session: Session = global_session
    ) -> None:
        """
        Creates a new password associated to the current api_key

        :param password:
        """
        req = PasswordRequest(password=password)
        cls._create(session=session, **req.dict())

    @classmethod
    def delete(
        cls, password: str, *, user_id: str = 'me', session: Session = global_session
    ) -> None:
        """
        Use this method to deactivate your current password

        :param password: Current password
        """
        req = PasswordRequest(password=password)
        url = f'{cls._resource}/{user_id}'
        session.delete(url, req.dict())
